#!/usr/bin/env python
#******************************************************************************
#**** Copyright (C) 2009, 2010                                             ****
#****   John Schneiderman <JohnMS@member.fsf.org>                          ****
#****                                                                      ****
#**** This program is free software: you can redistribute it and/or modify ****
#**** it under the terms of the GNU General Public License as published by ****
#**** the Free Software Foundation, either version 3 of the License, or    ****
#**** (at your option) any later version.                                  ****
#****                                                                      ****
#**** This program is distributed in the hope that it will be useful,      ****
#**** but WITHOUT ANY WARRANTY; without even the implied warranty of       ****
#**** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        ****
#**** GNU General Public License for more details.                         ****
#****                                                                      ****
#**** You should have received a copy of the GNU General Public License    ****
#**** along with this program.  If not, see <http://www.gnu.org/licenses/> ****
#******************************************************************************

"""
 IMPORTS
"""
from mediasong import MediaSong

import wx

import mediaparser as parser


class SearchWindow(wx.Panel):
    """ Interface to search for a song in the media library. """

    """
     ATTRIBUTES
    """
    # A list of the MediaSong objects displayed in the results panel.
    __searchResults = []
    # Is the width of the media column.
    __MEDIA_WIDTH = 50
    # Approximate width of the vertical scroll bar
    __VERTICAL_SCROLL_BAR = 25
    # Maximum number of results to display at once
    __MAXIMUM_DISPLAYED_RESULTS = 50
    # Current index position displayed
    __displayedResultsStartIndex = None

    """
     EVENT ID
    """
    # Identifier for the area where the user enters search terms.
    ID_SEARCH_ENTRY = wx.NewId()
    # Identifier for the result display.
    ID_SEARCH_RESULTS = wx.NewId() # Display for the results of a search
    # Search for audio media
    __SEARCH_AUDIO = wx.NewId()
    # Search for video media
    __SEARCH_VIDEO = wx.NewId()
    # Column ID for the media type in the results window
    __ID_MEDIA_COLUMN = 0
    # Column ID for the song title in the results window
    __ID_TITLE_COLUMN = 1
    # Column ID for the artist title in the results window
    __ID_ARTIST_COLUMN = 2
    # Previous search results page
    __ID_PREVIOUS = wx.NewId()
    # Next search results page
    __ID_NEXT = wx.NewId()
    # Search result page indicator
    __ID_PAGES = wx.NewId()

    def __init__(self, parent, iconPath, searchAudio=True, searchVideo=True):
        """ Default constructor

         wx.Window parent: is the parent window.
         boolean searchAudio: True if the user wants to search for audio media.
         boolean searchVideo: True if the user wants to search for video media.
        """
        wx.Panel.__init__(self, parent, wx.NewId())
        self.__createControls(searchAudio, searchVideo, iconPath)
        self.__bindEvents()
        self.__doLayout()

    def __createControls(self, searchAudio, searchVideo, iconPath):
        """ Create the controls for the search window.

         boolean searchAudio: True if the user wants to search for audio media.
         boolean searchVideo: True if the user wants to search for video media.
        """
        # Create Search Box
        self.__srchCtrlSearch = wx.SearchCtrl(self, self.ID_SEARCH_ENTRY, \
            style=wx.TE_PROCESS_ENTER)
        self.__srchCtrlSearch.ShowCancelButton(True)
        self.__srchCtrlSearch.ShowSearchButton(True)
        self.__srchCtrlSearch.SetDescriptiveText("Search Media Library")

        # Create Search Box Menu
        self.__mnuSearch = wx.Menu("Search Options")
        self.__mnuSearch.AppendCheckItem(self.__SEARCH_AUDIO, "Audio Media")
        self.__mnuSearch.Check(self.__SEARCH_AUDIO, searchAudio)
        self.__mnuSearch.AppendCheckItem(self.__SEARCH_VIDEO, "Video Media")
        self.__mnuSearch.Check(self.__SEARCH_VIDEO, searchVideo)
        self.__srchCtrlSearch.SetMenu(self.__mnuSearch)

        # Create Search Result Displays
        self.__lstCtlSearchResults = wx.ListCtrl(self, \
            self.ID_SEARCH_RESULTS, style=wx.LC_REPORT | wx.SUNKEN_BORDER | \
             wx.LC_HRULES | wx.LC_VRULES)
        self.__lstCtlSearchResults.InsertColumn(self.__ID_MEDIA_COLUMN, \
            "Media", self.__MEDIA_WIDTH)
        self.__lstCtlSearchResults.InsertColumn(self.__ID_TITLE_COLUMN, \
            "Title")
        self.__lstCtlSearchResults.InsertColumn(self.__ID_ARTIST_COLUMN, \
            "Artist")

        # Large list navigation
        self.__btmpBttnPrevious = wx.BitmapButton(self, self.__ID_PREVIOUS, \
            wx.Bitmap(iconPath + '/prev.png'))
        self.__btmpBttnPrevious.Enable(False)
        self.__spnCtrlPage = wx.SpinCtrl(self, self.__ID_PAGES, \
            style=wx.TE_READONLY)
        self.__spnCtrlPage.SetRange(0, 0)
        self.__btmpBttnNext = wx.BitmapButton(self, self.__ID_NEXT, \
            wx.Bitmap(iconPath + '/next.png'))
        self.__btmpBttnNext.Enable(False)

    def __bindEvents(self):
        """ Connects all the needed events.

         Connects each of the control objects created to their corresponding
           methods.
        """
        wx.EVT_LIST_INSERT_ITEM(self, self.ID_SEARCH_RESULTS, \
            self.__onListInsertItem)

        # Search Navigation Events
        wx.EVT_BUTTON(self, self.__ID_PREVIOUS, self.__onPreviousSearch)
        wx.EVT_BUTTON(self, self.__ID_NEXT, self.__onNextSearch)
        wx.EVT_SPINCTRL(self, self.__ID_PAGES, self.__onPageChanged)

        # File Informaiton Events
        wx.EVT_LIST_ITEM_RIGHT_CLICK(self, self.ID_SEARCH_RESULTS, \
            self.__onFileInformaiton)

    def __doLayout(self):
        """ Creates a visually pleasing look for the controls.

         All the control objects are placed on the window.
        """

        # Search Entry Layout
        self.__bxSzrSearchEntry = wx.BoxSizer(wx.HORIZONTAL)
        self.__bxSzrSearchEntry.Add(self.__srchCtrlSearch, 1, wx.EXPAND | \
            wx.ALL | wx.FIXED_MINSIZE, 3)

        # Search Navigation Layout
        self.__bxSzrSearchNavigation = wx.BoxSizer(wx.HORIZONTAL)
        self.__bxSzrSearchNavigation.AddSpacer(5)
        self.__bxSzrSearchNavigation.Add(self.__spnCtrlPage, 0, wx.ALL, 3)
        self.__bxSzrSearchNavigation.AddSpacer(5)
        self.__bxSzrSearchNavigation.Add(self.__btmpBttnPrevious, 1, wx.EXPAND | \
            wx.ALL | wx.FIXED_MINSIZE, 3)
        self.__bxSzrSearchNavigation.AddSpacer(5)
        self.__bxSzrSearchNavigation.Add(self.__btmpBttnNext, 1, wx.EXPAND | \
            wx.ALL | wx.FIXED_MINSIZE, 3)

        # Search Result Layout
        self.__bxSzrSearchResults = wx.BoxSizer(wx.VERTICAL)
        self.__bxSzrSearchResults.Add(self.__lstCtlSearchResults, 1, \
            wx.EXPAND | wx.ALL, 3)

        # Create Interface
        self.__bxSzrInterface = wx.BoxSizer(wx.VERTICAL)
        self.__bxSzrInterface.Add(self.__bxSzrSearchEntry, 0, wx.EXPAND | \
            wx.ALL)
        self.__bxSzrInterface.AddSpacer(5)
        self.__bxSzrInterface.Add(self.__bxSzrSearchResults, 1, wx.EXPAND | \
            wx.ALL)
        self.__bxSzrInterface.Add(self.__bxSzrSearchNavigation, 0)
        self.SetSizer(self.__bxSzrInterface)

    def searchTerms(self):
        """ Accessor to the search terms.

         return string: the search terms entered by the user.
        """
        return self.__srchCtrlSearch.GetValue()

    def searchForAudios(self):
        """ Accessor to the search criteria

         return boolean: True if the user wants to search for audio media.
        """
        return self.__mnuSearch.IsChecked(self.__SEARCH_AUDIO)

    def searchForVideos(self):
        """ Accessor to the search criteria

         return boolean: True if the user wants to search for video media.
        """
        return self.__mnuSearch.IsChecked(self.__SEARCH_VIDEO)

    def clearSearchTerms(self):
        """ Clears the search terms. """
        self.__srchCtrlSearch.SetValue("")

    def selectedSongs(self):
        """ Accessor to the currently selected songs

         return list[MediaSong]: each of the songs selected by the user.
        """
        selection = []
        current = -1 # Current index
        while True:
            next = self.__getNextSelectedSong(current)
            if next == -1:
                return selection

            selection.append(self.__searchResults[next + \
                self.__displayedResultsStartIndex])
            current = next

    def __getNextSelectedSong(self, current=-1):
        """ Get the next selected song in the results display.

         int current: is the current index.
         return int: the index of the next selected item, or -1 when no more
           selections are left.
        """
        return self.__lstCtlSearchResults.GetNextItem(current, \
            wx.LIST_NEXT_ALL, wx.LIST_STATE_SELECTED)

    def clearSearchResults(self):
        """ Clears the display result window """
        del self.__searchResults[:]
        self.__lstCtlSearchResults.DeleteAllItems()
        self.__displayedResultsStartIndex = None

    def addSearchResult(self, result):
        """ Adds a search result to display for the user.

         MediaSong result: is the song to add.
        """
        self.__searchResults.append(result)
        index = len(self.__searchResults) - 1
        if self.__displayedResultsStartIndex is None:
            self.__addResultItem(index, result)
            self.__displayedResultsStartIndex = 0
        elif len(self.__searchResults) <= self.__MAXIMUM_DISPLAYED_RESULTS:
            self.__addResultItem(index, result)
        self.__updateNavigationButtons()

    def __addResultItem(self, index, song):
        """ Adds the song from the search results to the display. """

        # Title Information
        item = wx.ListItem()
        item.SetId(index)
        item.SetColumn(self.__ID_TITLE_COLUMN)
        item.SetText(song.title)
        self.__lstCtlSearchResults.InsertItem(item)
        # Artist Information
        item = wx.ListItem()
        item.SetId(index)
        item.SetColumn(self.__ID_ARTIST_COLUMN)
        item.SetText(song.artist)
        self.__lstCtlSearchResults.SetItem(item)
        # Video Information
        item = wx.ListItem()
        item.SetId(index)
        item.SetColumn(self.__ID_MEDIA_COLUMN)
        if song.blIsVideo:
            item.SetText("Video")
        else:
            item.SetText("Audio")
        self.__lstCtlSearchResults.SetItem(item)

    def __onListInsertItem(self, event):
        """ Handles the column width display as each item is inserted.

         wx.Event event: is the event object (Not Used)
        """
        self.__adjustResultColumns()

    def __adjustResultColumns(self):
        """ Adjusts the width of the result columns. """
        width = self.__bxSzrInterface.GetSize().GetWidth() - \
            self.__MEDIA_WIDTH - self.__VERTICAL_SCROLL_BAR
        self.__lstCtlSearchResults.SetColumnWidth(self.__ID_TITLE_COLUMN, \
            width / 2)
        self.__lstCtlSearchResults.SetColumnWidth(self.__ID_ARTIST_COLUMN, \
            width / 2)
        self.__lstCtlSearchResults.SetColumnWidth(self.__ID_MEDIA_COLUMN, \
            self.__MEDIA_WIDTH)

    def __onPreviousSearch(self, event):
        """ Adjusts the result display to the previous self.__MAXIMUM_DISPLAYED_RESULTS searches. """
        self.__displayedResultsStartIndex -= self.__MAXIMUM_DISPLAYED_RESULTS
        self.__displaySearchResultsFromIndex(self.__displayedResultsStartIndex)
        self.__updateNavigationButtons()

    def __onNextSearch(self, event):
        """ Adjusts the result display to the next self.__MAXIMUM_DISPLAYED_RESULTS searches. """
        self.__displayedResultsStartIndex += self.__MAXIMUM_DISPLAYED_RESULTS
        self.__displaySearchResultsFromIndex(self.__displayedResultsStartIndex)
        self.__updateNavigationButtons()

    def __displaySearchResultsFromIndex(self, startIndex):
        """ Display's the results from a starting index. """
        self.__lstCtlSearchResults.DeleteAllItems()
        resultIndex = 0
        if (self.__displayedResultsStartIndex + \
                self.__MAXIMUM_DISPLAYED_RESULTS) < len(self.__searchResults):
            endIndex = (self.__MAXIMUM_DISPLAYED_RESULTS + startIndex)
        else:
            endIndex = len(self.__searchResults)
        for index in range(startIndex, endIndex):
            self.__addResultItem(resultIndex, self.__searchResults[index])
            index += 1
            resultIndex += 1

    def __updateNavigationButtons(self):
        """ Updates the navigation buttons based on the search results. """
        if (self.__displayedResultsStartIndex + \
                self.__MAXIMUM_DISPLAYED_RESULTS) > len(self.__searchResults):
            self.__btmpBttnNext.Enable(False)
        elif len(self.__searchResults) > self.__MAXIMUM_DISPLAYED_RESULTS:
            self.__btmpBttnNext.Enable(True)
        else:
            self.__btmpBttnNext.Enable(False)

        if self.__displayedResultsStartIndex < 1:
            self.__btmpBttnPrevious.Enable(False)
        else:
            self.__btmpBttnPrevious.Enable(True)

        if self.__displayedResultsStartIndex is not None:
            self.__spnCtrlPage.SetRange(1, len(self.__searchResults) / \
                self.__MAXIMUM_DISPLAYED_RESULTS + 1)
            self.__spnCtrlPage.SetValue(self.__displayedResultsStartIndex / \
                self.__MAXIMUM_DISPLAYED_RESULTS + 1)

    def __onPageChanged(self, event):
        """ Hangles the changes in pages from the user  """
        currentPage = self.__displayedResultsStartIndex / \
                self.__MAXIMUM_DISPLAYED_RESULTS + 1
        newPage = self.__spnCtrlPage.GetValue()
        if currentPage > newPage:
            self.__onPreviousSearch(event)
        elif currentPage < newPage:
            self.__onNextSearch(event)
        else:
            return

    def __onFileInformaiton(self, event):
        """ Handles the display of file informaiton """
        if self.__displayedResultsStartIndex is None:
            return
        song = self.__searchResults[self.__getNextSelectedSong() + \
                self.__displayedResultsStartIndex]
        songInfo = parser.MediaParser(song)
        if songInfo.hasInformation():
            knownInformation = songInfo.information()
            info = ""
            for key in knownInformation.keys():
                info += "  %s: %s\n" % (key, knownInformation[key])
            dlg = wx.MessageDialog(self, info,"Media Informaiton", \
                wx.ICON_INFORMATION | wx.OK)
        else:
            dlg = wx.MessageDialog(self, "Unable to locate song %s." % song, \
                "Media Informaiton Error", wx.ICON_INFORMATION | wx.OK)
        dlg.ShowModal()
